/*
 * ARDUINO-DATENUEBERTRAGUNG AN PROFILAB EXPERT 4.0
 * 
 * 24/06/2015 - W. Kleider
 * 
 * ACHTUNG: Die ProfiLab-Schnittstelle splittet den gesendeten String am Zeichen "\t" auf.
 * Das Senden von weniger Daten als in ProfiLab eingestellt ist problemlos möglich (z.B. nur 2 statt 4 Kanäle).
 * Das Senden von mehr Daten ist hingegen nicht möglich. ProfiLab trennt nach dem letzten Kanal nicht mehr am TAB auf.
 * 
 * Die analogen Messdaten werden unverarbeitet weitergegeben (0..1023).
 *
*/

// Pinbelegung am Arduino
const int inPin1 = A3;     // Eingangspin (analog!) des ersten Hallsensors - muss mit Spannungsteiler fuer Wechselspannung vorbereitet werden 
const int inPin2 = A5;     // Eingangspin (analog!) des zweiten Hallsensors - muss mit Spannungsteiler fuer Wechselspannung vorbereitet werden 

// Messvariablen
int inValue1 = 0;          // Wert des ersten Hallpins
int offsetValue1 = 0;      // Offset-Value des ersten Hallpins
int inValue2 = 0;          // Wert des zweiten Hallpins
int offsetValue2 = 0;      // Offset-Value des zweiten Hallpins

// Standard-Einstellungen fuer die Messung - koennen ueber ProfiLab eingestellt werden
int refreshRate = 20;    // Abstand zwischen zwei Messpunkten in ms. Muss mindestens 50 sein. Standard: 50

// Variablen fuer Programmablauf (Serielle Kommunikation)
unsigned long currentTime = 0; // Zeitmarke fuer Messung
unsigned long startTime = 0;   // Beginn einer laufenden Messung
String inputString = "";       // Zwischenspeicher fuer Nachrichten des ProfiLab-Programms
bool inputStringComplete = false; // Pruefung der Vollstaendigkeit dieser Nachrichten
bool sendData = false;         // Datensendung erwuenscht - wird von ProfiLab gesteuert

void setup() {
  Serial.begin(115200);
  Serial.println("");
  
  currentTime = millis();
}

void loop() {
  // Speichere den Zeitpunkt des Beginns dieser Schleife
  currentTime = millis(); 
  
  // Pruefe, ob es neue Befehle vom ProfiLab-Programm gibt
  if (inputStringComplete) {
    String messageReceived = inputString;
    if (messageReceived.startsWith("START")) {
      sendData = true;
      startTime = currentTime;
    }
    else if (messageReceived.startsWith("STOP")) {
      sendData = false;
    }
    else if (messageReceived.startsWith("SET POSITION=0")) {
      // Setze Offsets fuer beide Pins - erlaubt Messung der Wechselspannung am Spannungsteiler
      offsetValue1 = analogRead(inPin1);
      offsetValue2 = analogRead(inPin2);
    }
    else if (messageReceived.startsWith("SET REFRESH=")) {
      refreshRate = messageReceived.substring(12).toInt();
      
      // Falls die gewuenschte RefreshRate zu klein ist (<50), setze sie auf 50.
      if (refreshRate < 50) {
        refreshRate = 50;
      }
    }
    else if (messageReceived.startsWith("RESET")) {
      delay(1000);
      software_Reset();
    }
    inputString = "";
    inputStringComplete = false;
  }
  
  /*
    Falls Datensendung erwuenscht, fuehre folgenden Block aus.
  */
  if (sendData) {
    
    // Messung der Eingangsspannungen, abzueglich Offset
    inValue1 = analogRead(inPin1) - offsetValue1;
    inValue2 = analogRead(inPin2) - offsetValue2;
    
    // Erzeugen der Mitteilung an Arduino
    String returnMessage = "";
    returnMessage += currentTime-startTime;
    returnMessage += "\t";
    returnMessage += inValue1;
    returnMessage += "\t";
    returnMessage += inValue2;
    
    // Senden der Mitteilung an ProfiLab
    Serial.println(returnMessage);
    
  } // Ende des Senden-Blocks

  
  // Warte, bis die Ruhezeit vergangen ist.
  while (millis() < (currentTime + refreshRate))
  {
  }
}

// Falls es neue Mitteilungen auf der Seriellen Schnittstelle gibt, lies diese aus.
void serialEvent() {
  while (Serial.available()) {
    char inputChar = (char)Serial.read();
    inputString += inputChar;
    
    if(inputChar=='\n') {
      inputStringComplete = true;
    }
  }
}

// Fuehre einen Software-Reset des Arduino durch.
// Restarts program from beginning but does not reset the peripherals and registers
void software_Reset() { 
  asm volatile ("  jmp 0");
}
